@doc raw"""
    read_data(ens::String, g1::String="G5", g2::String="G5")
Given the ensemble id and the Dirac structure, this method reads the data in path_data and
returns a CData object. Replicas are automatically detected and loaded if present.
"""
function read_data(ens::String, g1::String="G5", g2::String="G5"; legacy::Bool=false)
    path = joinpath(path_data, ens)
    rep = filter(x->occursin("mesons.dat", x), readdir(path, join=true))
    if length(rep)!=0    
        length(rep)==1 ? (return read_mesons(rep[1], g1, g2, legacy=legacy)) : (return read_mesons(rep, g1, g2, legacy=legacy))
    else
        error("mesons.dat file not found for ensemble ", ens, " in path ", path)
    end
end
function read_rw(ens::String)
    path = joinpath(path_data, ens)
    rep = filter(x->occursin("ms1.dat", x), readdir(path, join=true))
    if length(rep)!=0
        try
            length(rep) == 1 ? (return read_ms1(rep[1])) : (return read_ms1.(rep)) 
        catch
            length(rep) == 1 && length(rep)!=0 ? (return read_ms1(rep[1], v="1.4")) : (return read_ms1.(rep, v="1.4")) 
        end
    else
        error("ms1.dat file not found for ensemble ", ens, " in path ", path)
    end
end
function read_t0(ens::String)
    path = joinpath(path_data, ens)
    rep = filter(x->occursin("ms.dat", x), readdir(path, join=true))
    if length(rep)!=0
        length(rep) == 1 ? (return read_ms(rep[1])) : (return read_ms.(rep)) 
    else
        error("ms.dat file not found for ensemble ", ens, " in path ", path)
    end
end
function get_corr(ens::EnsInfo, g1::String="G5", g2::String="G5"; rw::Bool=false, legacy::Bool=false)
    aux = read_data(ens.id, g1, g2, legacy=legacy)
    !rw ? obs = corr_obs.(aux, L=ens.L) : obs = corr_obs.(aux, L=ens.L, rw=read_rw(ens.id))
    return obs
end
function get_t0(ens::EnsInfo; rw::Bool=false, pl::Bool=false)
    data = read_t0(ens.id)
    plat = [40,160] 
    !rw ? t0 = comp_t0(data, plat, L=ens.L, pl=pl) : t0 = comp_t0(data, plat, L=ens.L, pl=pl, rw=read_rw(ens.id))
    return t0
end
function get_mu(mu_list::Vector{Vector{Float64}}, deg::Bool)
    mu_sorted = unique(sort(minimum.(mu_list)))

    mul = mu_sorted[1]
    deg ? mus = 0.0 : mus = mu_sorted[2]
    muh = unique(maximum.(mu_list))
    muh = filter(x-> x > mul && x > mus, muh)

    return mul, mus, muh
end
function get_ll(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] == mu[2] #l-l
            return obs[k]
        end
    end
end
function get_ls(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mus in mu  #l-l
            return obs[k]
        end
    end
end
function get_lh(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_lh = Vector{uwreal}(undef, 0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] != mu[2] && !(mus in mu) #l-h
            push!(obs_lh, obs[k])
        end
    end
    return obs_lh
end
function get_ss(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mus in mu && mu[1] == mu[2] #s-s
            return obs[k]
        end
    end
end
function get_sh(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_sh = Vector{uwreal}(undef, 0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mus in mu && mu[1] != mu[2] && !(mul in mu)#s-h
            push!(obs_sh, obs[k])
        end
    end
    return obs_sh
end
function get_hh(mu_list, obs::Vector{uwreal}, deg::Bool)
    mul, mus, muh = get_mu(mu_list, deg)
    obs_hh = Vector{uwreal}(undef,0)
    for k = 1:length(mu_list)
        mu = mu_list[k]
        for i =1:length(muh)
            if muh[i] in mu && mu[1] == mu[2]#h-h
                push!(obs_hh, obs[k])
            end
        end
    end
    return obs_hh
end
function get_meff(obs::Vector{juobs.Corr}, ens::EnsInfo; pl::Bool=false)
    mu_list  = getfield.(obs, :mu)
    plat = select_plateau(ens, mu_list)
    return meff.(obs, plat, pl=pl)
end
function get_f(obs::Vector{juobs.Corr}, m::Vector{uwreal}, ens::EnsInfo; pl::Bool=false)
    mu_list = getfield.(obs, :mu)
    plat = select_plateau(ens, mu_list)
    return dec_const_pcvc.(obs, plat, m, pl=pl)
end
function select_plateau(ensinf::EnsInfo, mu_list)
    ens =ensinf.id
    deg = ensinf.deg
    mul, mus, muh = get_mu(mu_list, deg)
    f = readdlm(path_plat)
    head = String.(f[:, 1])

    delim = findall(x-> occursin("#", x ), head)
    edelim = findfirst(x-> occursin(ens, x), head)
    cdelim = findfirst(x-> x.== edelim, delim)

    del = delim[cdelim]+1 : delim[cdelim+1]-1
    #del = delim
    plat = Vector{Vector{Int64}}(undef, 0)
    #plat = Dict()
    plat_ = Int64.(f[del, 2:3])
    head_ = String.(f[del, 1])
    
    for k = 1:length(mu_list)
        mu = mu_list[k]
        if mul in mu && mu[1] != mu[2] && !(mus in mu)  #heavy-light
            n = findfirst(x-> occursin("lh", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]]  ) 
        elseif mul in mu && mu[1] == mu[2] #light-light
            n = findfirst(x-> occursin("ll", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )               
        elseif mul in mu && mus in mu#strange-light
            n = findfirst(x-> occursin("ls", x), head_)
            push!(plat,  [plat_[n, 1], plat_[n, 2]] )
        elseif mus in mu && mu[1] != mu[2] && !(mul in mu)#heavy-strange
            n = findfirst(x-> occursin("sh", x ), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        elseif mus in mu && mu[1] == mu[2] && !(mul in mu)#strange-strange
            n = findfirst(x-> occursin("ss", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        elseif !(mul in mu) && !(mus in mu) #heavy-heavy
            n = findfirst(x-> occursin("hh", x), head_)
            push!(plat, [plat_[n, 1], plat_[n, 2]] )
        end
    end
    return plat
end

function match_muc(muh, m_lh, m_sh, target)
    M = (2/3 .* m_lh .+ 1/3 .* m_sh) 
    par, chi2exp = lin_fit(muh, M)
    muh_target = x_lin_fit(par, target)
    return muh_target
end